import builtins

from PyQt5.Qsci import QsciScintilla, QsciLexerPython, QsciAPIs
from PyQt5.QtGui import QFontMetrics

from glia.widgets.editor.lexers import LexerBase


class PythonLexer(LexerBase):
    """
    Customized from QsciLexerPython to provide basic auto-completion.
    """
    def lock(self):
        """
        Sets the default properties for the Python lexer.
        """
        # Lexer Initialization
        lexer = QsciLexerPython(self.code_editor)
        lexer.setDefaultFont(self.font)
        self.code_editor.setLexer(lexer)

        # Auto Completion
        api = QsciAPIs(lexer)
        for var in dir(builtins):
            if not (var[0] == "_"):
                api.add(var)
        api.prepare()

        self.code_editor.setAutoCompletionThreshold(1)
        self.code_editor.setAutoCompletionSource(QsciScintilla.AcsAPIs)

        # Indentation
        self.code_editor.setIndentationWidth(4)

        # Font Settings
        font_metrics = QFontMetrics(self.font)
        self.code_editor.setMinimumSize(
            int(font_metrics.width("0" * 80)),
            0
        )



import sys
from pathlib import Path

from PyQt5.QtWidgets import QApplication

from glia.windows.main import MainWindow


def start(**kwargs):
    app = QApplication(sys.argv)

    # Load Theme
    with open(Path("../resources/themes/dracula.qss")) as qss:
        app_style = qss.read()
    app.setStyleSheet(app_style)

    # Create Main Window
    window = MainWindow()
    window.showMaximized()

    # Run & Exit
    sys.exit(app.exec_())


if __name__ == "__main__":
    start()



from PyQt5.QtWidgets import QTabWidget

from glia.widgets.editor import Editor


class EditorTabs(QTabWidget):
    def __init__(self, parent=None):
        """
        Generates tab with editor depending on the file and it's type
        selected.
        """
        super(EditorTabs, self).__init__(parent)

        # Tab Properties
        self.setTabsClosable(True)
        self.setMovable(True)

        # Default Editor
        self.editor = Editor(self)
        self.addTab(self.editor, "main.py")

        # Slots
        self.tabCloseRequested.connect(self.handle_tab_closed)

    def handle_tab_closed(self, index):
        self.removeTab(index)
